package flattens

import (
	"context"
	"encoding/json"
	"fmt"
	"strconv"

	"github.com/hashicorp/terraform-plugin-framework/diag"
	"github.com/hashicorp/terraform-plugin-framework/types"
	"github.com/hashicorp/terraform-plugin-log/tflog"
	"repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudbroker/disks"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/client"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/flattens"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/service/cloudbroker/disks/models"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/service/cloudbroker/disks/utilities"
)

// DiskReplicationResource flattens resource for disk.
// Return error in case resource is not found on the platform.
// Flatten errors are added to tflog.
func DiskReplicationResource(ctx context.Context, state *models.ResourceRecordDiskReplicationModel, c *client.Client) diag.Diagnostics {
	tflog.Info(ctx, "Start flattens.DiskReplicationresource")

	diags := diag.Diagnostics{}

	recordDisk, status, err := utilities.ResourceDiskReplicationCheckPresence(ctx, state, c)
	if err != nil {
		diags.AddError("Cannot get info about disk", err.Error())
		return diags
	}

	tflog.Info(ctx, "flattens.DiskReplicationresource: before flatten")

	diskAcl, _ := json.Marshal(recordDisk.ACL)

	*state = models.ResourceRecordDiskReplicationModel{
		DiskId:              state.DiskId,
		Name:                state.Name,
		SepID:               state.SepID,
		ReplicationId:       state.ReplicationId,
		Timeouts:            state.Timeouts,
		PoolName:            state.PoolName,
		Pause:               state.Pause,
		Reverse:             state.Reverse,
		Start:               state.Start,
		Detach:              state.Detach,
		Permanently:         state.Permanently,
		Id:                  types.StringValue(strconv.Itoa(int(recordDisk.Replication.DiskID))),
		ACL:                 types.StringValue(string(diskAcl)),
		BootPartition:       types.Int64Value(int64(recordDisk.BootPartition)),
		AccountID:           types.Int64Value(int64(recordDisk.AccountID)),
		AccountName:         types.StringValue(recordDisk.AccountName),
		Computes:            flattenRComputes(ctx, recordDisk.Computes),
		CreatedTime:         types.Int64Value(int64(recordDisk.CreatedTime)),
		DeletedTime:         types.Int64Value(int64(recordDisk.DeletedTime)),
		DeviceName:          types.StringValue(recordDisk.DeviceName),
		DiskPath:            types.StringValue(recordDisk.DiskPath),
		Description:         types.StringValue(recordDisk.Description),
		DestructionTime:     types.Int64Value(int64(recordDisk.DestructionTime)),
		GUID:                types.Int64Value(int64(recordDisk.GUID)),
		GID:                 types.Int64Value(int64(recordDisk.GID)),
		ImageID:             types.Int64Value(int64(recordDisk.ImageID)),
		Images:              flattens.FlattenSimpleTypeToList(ctx, types.StringType, recordDisk.Images),
		IQN:                 types.StringValue(recordDisk.IQN),
		Login:               types.StringValue(recordDisk.Login),
		Milestones:          types.Int64Value(int64(recordDisk.Milestones)),
		Order:               types.Int64Value(int64(recordDisk.Order)),
		Params:              types.StringValue(recordDisk.Params),
		ParentID:            types.Int64Value(int64(recordDisk.ParentID)),
		Passwd:              types.StringValue(recordDisk.Password),
		PCISlot:             types.Int64Value(int64(recordDisk.PCISlot)),
		Pool:                types.StringValue(recordDisk.Pool),
		PresentTo:           flattens.FlattenSimpleTypeToList(ctx, types.StringType, recordDisk.PresentTo),
		PurgeAttempts:       types.Int64Value(int64(recordDisk.PurgeAttempts)),
		PurgeTime:           types.Int64Value(int64(recordDisk.PurgeTime)),
		RealityDeviceNumber: types.Int64Value(int64(recordDisk.RealityDeviceNumber)),
		ReferenceID:         types.StringValue(recordDisk.ReferenceID),
		ResID:               types.StringValue(recordDisk.ResID),
		ResName:             types.StringValue(recordDisk.ResName),
		Role:                types.StringValue(recordDisk.Role),
		SepType:             types.StringValue(recordDisk.SEPType),
		Shareable:           types.BoolValue(recordDisk.Shareable),
		SizeMax:             types.Int64Value(int64(recordDisk.SizeMax)),
		SizeUsed:            types.Float64Value(recordDisk.SizeUsed),
		Snapshots:           replicationFlattenSnapshots(ctx, recordDisk.Snapshots),
		Status:              types.StringValue(recordDisk.Status),
		StatusReplication:   types.StringValue(*status),
		TechStatus:          types.StringValue(recordDisk.TechStatus),
		Type:                types.StringValue(recordDisk.Type),
		VMID:                types.Int64Value(int64(recordDisk.VMID)),
	}

	iotune := models.ResourceDiskReplicationIOTuneModel{
		ReadBytesSec:     types.Int64Value(int64(recordDisk.IOTune.ReadBytesSec)),
		ReadBytesSecMax:  types.Int64Value(int64(recordDisk.IOTune.ReadBytesSecMax)),
		ReadIOPSSec:      types.Int64Value(int64(recordDisk.IOTune.ReadIOPSSec)),
		ReadIOPSSecMax:   types.Int64Value(int64(recordDisk.IOTune.ReadIOPSSecMax)),
		SizeIOPSSec:      types.Int64Value(int64(recordDisk.IOTune.SizeIOPSSec)),
		TotalBytesSec:    types.Int64Value(int64(recordDisk.IOTune.TotalBytesSec)),
		TotalBytesSecMax: types.Int64Value(int64(recordDisk.IOTune.TotalBytesSecMax)),
		TotalIOPSSec:     types.Int64Value(int64(recordDisk.IOTune.TotalIOPSSec)),
		TotalIOPSSecMax:  types.Int64Value(int64(recordDisk.IOTune.TotalIOPSSecMax)),
		WriteBytesSec:    types.Int64Value(int64(recordDisk.IOTune.WriteBytesSec)),
		WriteBytesSecMax: types.Int64Value(int64(recordDisk.IOTune.WriteBytesSecMax)),
		WriteIOPSSec:     types.Int64Value(int64(recordDisk.IOTune.WriteIOPSSec)),
		WriteIOPSSecMax:  types.Int64Value(int64(recordDisk.IOTune.WriteIOPSSecMax)),
	}
	obj, diags := types.ObjectValueFrom(ctx, models.ResourceDiskReplicationIOTune, iotune)
	if diags != nil {
		tflog.Error(ctx, fmt.Sprint("Error flattens.ReplicationDiskresource iotune struct to obj", diags))
	}
	state.IOTune = obj

	itemReplication := models.ResourceItemReplicationModel{
		DiskID:       types.Int64Value(int64(recordDisk.Replication.DiskID)),
		PoolID:       types.StringValue(recordDisk.Replication.PoolID),
		Role:         types.StringValue(recordDisk.Replication.Role),
		SelfVolumeID: types.StringValue(recordDisk.Replication.SelfVolumeID),
		StorageID:    types.StringValue(recordDisk.Replication.StorageID),
		VolumeID:     types.StringValue(recordDisk.Replication.VolumeID),
	}

	obj, diags = types.ObjectValueFrom(ctx, models.ResourceItemReplication, itemReplication)
	if diags != nil {
		tflog.Error(ctx, fmt.Sprint("Error flattens.ReplicationDiskresource Replication struct to obj", diags))
	}
	state.Replication = obj

	tflog.Info(ctx, "flattens.ReplicationDiskresource: end flatten")
	return nil
}

func flattenRComputes(ctx context.Context, items map[string]string) types.List {
	tflog.Info(ctx, "Start flattenRComputes")
	tempSlice := make([]types.Object, 0, len(items))
	for id, name := range items {
		temp := models.ResourceReplicationItemComputeModel{
			ComputeId:   types.StringValue(id),
			ComputeName: types.StringValue(name),
		}
		obj, diags := types.ObjectValueFrom(ctx, models.ResourceReplicationItemCompute, temp)
		if diags != nil {
			tflog.Error(ctx, fmt.Sprint("Error flattenRComputes struct to obj", diags))
		}
		tempSlice = append(tempSlice, obj)
	}

	res, diags := types.ListValueFrom(ctx, types.ObjectType{AttrTypes: models.ItemCompute}, tempSlice)
	if diags != nil {
		tflog.Error(ctx, fmt.Sprint("Error flattenRComputes", diags))
	}

	tflog.Info(ctx, "End flattenRComputes")
	return res
}

func replicationFlattenSnapshots(ctx context.Context, snapshots disks.ListSnapshots) types.List {
	tflog.Info(ctx, "Start flattenSnapshots")
	tempSlice := make([]types.Object, 0, len(snapshots))
	for _, item := range snapshots {
		temp := models.ResourceReplicationItemSnapshotModel{
			GUID:        types.StringValue(item.GUID),
			Label:       types.StringValue(item.Label),
			ResID:       types.StringValue(item.ResID),
			SnapSetGUID: types.StringValue(item.SnapSetGUID),
			SnapSetTime: types.Int64Value(int64(item.SnapSetTime)),
			TimeStamp:   types.Int64Value(int64(item.Timestamp)),
		}
		obj, diags := types.ObjectValueFrom(ctx, models.ResourceReplicationItemSnapshot, temp)
		if diags != nil {
			tflog.Error(ctx, fmt.Sprint("Error flattenSnapshots struct to obj", diags))
		}
		tempSlice = append(tempSlice, obj)
	}

	res, diags := types.ListValueFrom(ctx, types.ObjectType{AttrTypes: models.ResourceReplicationItemSnapshot}, tempSlice)
	if diags != nil {
		tflog.Error(ctx, fmt.Sprint("Error flattenSnapshots", diags))
	}

	tflog.Info(ctx, "End flattenSnapshots")
	return res
}
